@** Algebraic operations. We now reach % -*- mode: cweb; mode: fold; -*-
the point where serious optimization begins: basic
algebraic operations are the heart of any matrix package, and if your basic operations suck your
entire package will suck too. Since we're defining generic interfaces, we can't possibly reach
state-of-the-art performance -- but we can get pretty close.
@c
/* Empty, waiting for |export| */
@ @(algebra.h@>=
#ifndef __MATH_ALGEBRA__
#define __MATH_ALGEBRA__ 1.0
#include <math.h> /* For |sqrt|. */
#include <math/math.h>
namespace math
{
    @<Algebraic operations@>@;
}
#endif
@ We begin our definitions with binary algebraic operators. Unary operators were defined already.
These are a little trickier than unary operators because we have structures: the matrix returned
can have a different type of structure than that of the arguments (for example, a symmetric
matrix times a symmetric matrix is not necessarily symmetric). We therefore start with scalar
operations because this problem does not arise.
@<Algebraic operations@>=
template<matrix_simple_template>
matrix<T,structure,storage> operator*(const matrix<T,structure,storage>& x,
    const T& y)
{
    matrix<T,structure,storage> result=x;
    return result*=y;
}
@ @<Algebraic operations@>=
template<matrix_simple_template>
matrix<T,structure,storage> operator*(const submatrix<matrix<T,structure,storage> >&x,
	const T& y)
{
    matrix<T,structure,storage> result=x;
    return result*=y;
}
@ @<Algebraic operations@>=
template<matrix_simple_template>
matrix<T,structure,storage> operator+(const matrix<T,structure,storage>& x,
	const matrix<T,structure,storage>& y)
{
    if(x.rows() != y.rows() || x.cols()!=y.cols()) throw error::dimension();
    matrix<T,structure,storage> result=x;
    return result+=y;
}
@q Saxpy @>
@ The first function we define is one that overwrites a matrix~$Y$ with the value of~$\alpha
x+y$. The name {\em saxpy\/} is of common use (it's used, for example, in Golub and Van~Loan and
in the BLAS package), so we keep it. Note that with this method the variable~$y$ is always
overwritten, although some uses can be visually misleading. Also, it is the programmer who has to
be sure that the structure of~$y$ and $\alpha x+y$ is the same (or compatible), since~$y$ is
overwritten.
@<Algebraic operations@>=
template<class T,submatrix_template(X),submatrix_template(Y)>@;
SUBMY& saxpy(const T& a, const SUBMX& x, SUBMY* y)
{
    if(y->rows() != x.rows() || y->cols() != x.cols())
	@:matrix}{rows@>@:matrix}{cols@>
	@:submatrix}{rows@>@:submatrix}{cols@>
        throw error::dimension();@;

    for(index j=1;j<=y->cols();++j) for(index i=1;i<=y->rows();++i)
            y->set(i,j,a*x(i,j)+y->get(i,j));
    return *y;
}
@q@>
@q Dot-like @>
@ Next we define the dot product. This method only makes sense for vectors, and we test the
dimensions before procceeding.
@<Algebraic operations@>=
template<submatrix_template(X),submatrix_template(Y)>@;
typename SUBMX::element_type dot(const SUBMX &x, const SUBMY& y)
{
    if(x.cols()!=1 || y.cols()!=1 || x.rows()!=y.rows())
	@:matrix}{rows@>@:matrix}{cols@>
	@:submatrix}{rows@>@:submatrix}{cols@>
	throw error::dimension();

    typename SUBMX::element_type result=0;
    for(index i=1;i<=x.rows();++i) result += x(i)*y(i);
    return result;
}
@ Sometimes we want to multiply a row vector by a column vector. We call this operation the
``transposed |dot|'' operation. Here~$x$ must be a row vector.
@<Algebraic operations@>=
template<submatrix_template(X),submatrix_template(Y)>@;
typename SUBMX::element_type tdot(const SUBMX &x, const SUBMY& y)
{
    if(x.rows()!=1 || y.cols()!=1 || x.cols()!=y.rows())
	@:matrix}{rows@>@:matrix}{cols@>
	@:submatrix}{rows@>@:submatrix}{cols@>
	throw error::dimension();@;

    typename SUBMX::element_type result=0;
    for(index i=1;i<=x.cols();++i) result += x(1,i)*y(i);
    return result;
}
@ And, just to make you nervous, sometimes we want to take the ``dot product'' of two row
vectors. We call this operation the ``transposed-transposed |dot|'' operation. Here~$x$ and~$y$
must be row vectors.
@<Algebraic operations@>=
template<submatrix_template(X),submatrix_template(Y)>@;
typename SUBMX::element_type ttdot(const SUBMX &x, const SUBMY& y)
{
    if(x.rows()!=1 || y.rows()!=1 || x.cols()!=y.cols())
	@:matrix}{rows@>@:matrix}{cols@>
	@:submatrix}{rows@>@:submatrix}{cols@>
	throw error::dimension();@;

    typename SUBMX::element_type result=0;
    for(index i=1;i<=x.cols();++i) result += x(i)*y(i);
    return result;
}
@ Instead of defining the inner product of a vector we define the 2-norm operator. Since we're
calling the |dot| function we don't need to check dimensions here.
@<Algebraic operations@>=
template<submatrix_template(X)>@;
typename SUBMX::element_type norm2(const SUBMX& x)
{
    return(sqrt(dot(x,x)));
}
@q@>
@q Ax-like @>
@ The following method returns the result of the operation~$Ax$, where~$x$ is a vector or a
matrix. The third argument is the destination matrix.
@<Algebraic operations@>=
template<submatrix_template(A),submatrix_template(X),matrix_simple_template>@;
matrix<T,structure,storage>& 
axmul(const SUBMA& A,const SUBMX& x,matrix<T,structure,storage>* dest)
{
    index n=A.rows();
    index m=x.cols();
    index p=x.rows();
    @:matrix}{rows@>@:matrix}{cols@>
    @:submatrix}{rows@>@:submatrix}{cols@>

    if(A.cols() != p) throw error::dimension();@;
    
    dest->resize(n,m);
    for(index i=1;i<=n;++i)
	for(index j=1;j<=m;++j)
	{
	    T aux=0.0;
	    for(index k=1;k<=p;++k) aux+=A(i,k)*x(k,j);
	    dest->entry(i,j)=aux;
	}
    return *dest;
}
@ And this one returns the result of the operation~$A^Tx$, where~$x$ can be a matrix. The third
argument is the destination matrix.
@<Algebraic operations@>=
template<submatrix_template(A),submatrix_template(X),matrix_simple_template>@;
matrix<T,structure,storage>&
atxmul(const SUBMA& A, const SUBMX& x, matrix<T,structure,storage>* dest)
{
    index n=A.cols();@:matrix}{cols@>@:submatrix}{cols@>
    index m=x.cols();
    index p=x.rows();@:matrix}{rows@>@:submatrix}{rows@>

    if(A.rows()!=p) throw error::dimension();

    dest->resize(n,m);

    for(index i=1;i<=n;++i)
	for(index j=1;j<=m;++j)
	{
	    T aux=0.0;
	    for(index k=1;k<=p;++k) aux+=A(k,i)*x(k,j);
	    dest->entry(i,j)=aux;
	}
    return *dest;
}
@q@>
@q gaxpy @>
@ Next we present the generalized saxpy. In this method, the value~$a$ of |saxpy| is
substituted by a matrix, that is, now we have~$Y=AX+Y$. This method is normally heavily
used, and it involves a matrix multiplication -- it's a target of serious optimization in
some libraries. Our generic version must be, well, generic. It shouldn't be much worse than
routines for dense and unstructured matrices, however.
@<Algebraic operations@>=
template<submatrix_template(A),submatrix_template(X),submatrix_template(Y)>@;
SUBMY& gaxpy(const SUBMA& A, const SUBMX& x, SUBMY* y)
{
    if(y->rows() != A.rows() || y->cols() != x.cols() || x.rows() != A.cols())
	@:matrix}{rows@>@:matrix}{cols@>
	@:submatrix}{rows@>@:submatrix}{cols@>
        throw error::dimension();@;

    for(index j=1;j<=y->cols();++j)
        for(index i=1;i<=y->rows();++i)@/
	    y->set(i,j,y->get(i,j)+tdot(A.subm(i,i,1,A.cols()),x.subm(1,x.rows(),j)));
    return *y;
}
@ We also define a method in case what we want to do is to compute~$Y=A^TX+Y$.
@<Algebraic operations@>=
template<submatrix_template(A),submatrix_template(X),submatrix_template(Y)>@;
SUBMY& gatxpy(const SUBMA& A,const  SUBMX& x, SUBMY* y)
{
    if(y->rows() != A.cols() || y->cols() != x.cols() || x.rows() != A.rows())
	@:matrix}{rows@>@:matrix}{cols@>
	@:submatrix}{rows@>@:submatrix}{cols@>
        throw error::dimension();@;

    for(index j=1;j<=y->cols();++j)
        for(index i=1;i<=y->rows();++i)@/
	    y->set(i,j,y->get(i,j)+dot(A.subm(1,A.rows(),i),x.subm(1,A.rows(),j)));
    return *y;
}
@q@>
@q Outer products @>
@ The outer product operation (returns~$X\cdot X^T$) is sometimes useful.
@<Algebraic operations@>=
template<matrix_simple_template,submatrix_template(X)>@;
matrix<T,structure,storage>& outerp(const SUBMX& x, matrix<T,structure,storage>* dest)
{
    @:matrix}{rows@>@:matrix}{cols@>
    @:submatrix}{rows@>@:submatrix}{cols@>
    dest->resize(x.rows(),x.rows());
    T aux;

    for(index i=1;i<=x.rows();++i)
	for(index j=i;j<=x.rows();++j)
	{
	    aux=ttdot(x.subm(i,i,1,x.cols()),x.subm(j,j,1,x.cols()));
	    dest->entry(i,j)=aux;
	    dest->entry(j,i)=aux;
	}
    return *dest;
}
@ And just to facilitate, we enable the operation~$X^T\cdot X$ too.
@<Algebraic operations@>=
template<matrix_simple_template,submatrix_template(X)>@;
matrix<T,structure,storage>& outterp(const SUBMX& x, matrix<T,structure,storage>* dest)
{
    @:matrix}{rows@>@:matrix}{cols@>
    @:submatrix}{rows@>@:submatrix}{cols@>
    dest->resize(x.cols(),x.cols());
    T aux;

    for(index i=1;i<=x.cols();++i)
	for(index j=i;j<=x.cols();++j)
	{
	    aux=dot(x.subm(1,x.rows(),i,i),x.subm(1,x.rows(),j,j));
	    dest->entry(i,j)=aux;
	    dest->entry(j,i)=aux;
	}
    return *dest;
}
@ And finally a function to compute~$xy^T+yx^T$, where~$x$ and~$y$ are vectors.
@<Algebraic operations@>=
template<submatrix_template(Y),submatrix_template(X),matrix_simple_template>@;
matrix<T,structure,storage>& xyyx(const SUBMX& x,const SUBMY& y, matrix<T,structure,storage>* dest)
{
    if(y.cols()!=1 || x.cols()!=1 || x.rows()!=y.rows()) throw error::dimension();
    index n=x.rows();
    dest->resize(n,n);
    for(index i=1;i<=n;++i)
        for(index j=1;j<=n;++j)
	    dest->entry(i,j)=x(i)*y(j)+x(j)*y(i);
    return *dest;
}
@ Now to the outer product update. The function overwrites the matrix~$A$ with~$A+\beta xy^T$.
@<Algebraic operations@>=
template<submatrix_template(A),submatrix_template(X),submatrix_template(Y)>
SUBMA& outerp_update(SUBMA* A, typename SUBMA::element_type beta,
                     const SUBMX& x, const SUBMY& y)
{
    if(y.cols()!=1 || x.cols()!=1 || A->rows()!=x.rows() || A->cols()!=y.rows())
        throw error::dimension();
    index n=A->rows();
    index m=A->cols();
    for(index j=1;j<=m;++j)
        for(index i=1;i<=n;++i)
	    A->set(i,j,A->get(i,j)+beta*x(i)*y(j));
    return *A;
}
@q@>
