@* The log barrier function. Perhaps the most common, its value is $-\log(-f(x))$.
@c /* Empty, waiting for |export| */
@ @(barrier/log.h@>=
#ifndef __MATH_LOG_BARRIER__
#define __MATH_LOG_BARRIER__
#include <math/barrierbase.h>
namespace math
{
    @[namespace barrier@]
    {
	template<class T>
	@[class log : public base<T>@]
	{
	    matrix<T,unstructured,dense> aux,jac;
	    matrix<T,symmetric,dense> dense_aux;
	    matrix<T,symmetric,sparse> sparse_aux;
	public:@;
	    @<Log barrier methods@>@;
	};
    }
}
#endif
@ @<Log barrier methods@>=
log(functional::base<T>* newf)@+ {@+ f(newf);@+ }
log(function::base<T>* newf)@+ {@+ f(newf);@+ }
@ Remember that we have to take into account that an auxiliary variable may be present. In that
case, we first get the value of~$t$, resize~$x$ and get the value and the gradient of the
function. Resizing is not a big burden on performance because its optimized so that normally no
memory allocation is necessary.
@<Log barrier methods@>=
T eval(const matrix<T,unstructured,dense>& x)
{
    T result=0,t=0;
    matrix<T,unstructured,dense> X=x; /* We may resize |x|. */
    if(has_t)
    {
        t=X(X.rows());@:matrix}{rows@>
	X.resize(X.rows()-1,1);@:matrix}{resize@>
    }
    if(fctnal)
    {
        result=fctnal->eval(X)-t;@:functional}{eval@>
	if(result>=0) throw error::domain();
        return -::log(-result);
    }
    fct->eval(X,aux);@:function}{eval@>
    for(index i=1;i<=aux.rows();++i)@:matrix}{rows@>
    {
        if(aux(i)-t>=0) throw error::domain();
	result-=::log(t-aux(i));
    }
    return result;
}
@ For the logarithm, the dual variable is~$-1/(\mu f(x))$, so we return~$-1/f(x)$.
@<Log barrier methods@>=
T dual(const matrix<T,unstructured,dense>& x)@+ {@+ return -1/eval(x);@+ }
T dual_times_f(const matrix<T,unstructured,dense>&)@+ {@+ return T(-1.0);@+}
@ Now to the gradient: for a functional~$f$ we have $\nabla -\log(-f)=-(\nabla f)/f$. By our
definition for a function we have the sum~$-\sum_i(\nabla f_i)/f_i$.
@<Log barrier methods@>=@!@:barrier::log}{grad@>
matrix<T,unstructured,dense>& grad(const matrix<T,unstructured,dense>&x,
				   matrix<T,unstructured,dense>& dest)
{
    T t=0;
    matrix<T,unstructured,dense> X=x; /* We may resize |x|. */
    if(has_t)
    {
        t=X(X.rows());@:matrix}{rows@>
	X.resize(X.rows()-1,1);@:matrix}{resize@>
    }
    if(fctnal) 
    {
	fctnal->grad(X,dest);@:functional}{grad@>
	if(has_t)
	{
	    dest.resize(X.rows()+1,1);
	    dest.entry(dest.rows())=T(-1);@:matrix}{entry@>
	}
	dest/=t-fctnal->eval(X);@:functional}{eval@>@:matrix}{/=@>
	return dest;
    }
    fct->eval(X,aux);@:function}{eval@>
    fct->jacobian(X,dest);@:function}{jacobian@>
    if(has_t)
    {
        dest.resize(dest.rows(),dest.cols()+1);
	dest.subm(1,dest.rows(),dest.cols())=matrix<T,unstructured,dense>(dest.rows(),1,T(-1));
    }
    for(index i=1;i!=aux.rows();++i)@:matrix}{rows@>
    {
        dest.subm(i,i,1,dest.cols())/=t-aux(i);@:matrix}{subm@>
        if(i!=1) dest.subm(1,1,1,dest.cols())+=dest.subm(i,i,1,dest.cols());
    }
    dest=transpose(dest.subm(1,1,1,dest.cols()));
    return dest;
}
@ Finally the Hessian: for a functional we have~$\nabla^2 -\log(-f)=(\nabla
f\nabla^Tf)/f^2-(\nabla f)^2/f$.
@<Log barrier methods@>=@!@:barrier::log}{hess@>
matrix<T,symmetric,dense> &hess(const matrix<T,unstructured,dense>&x, matrix<T,symmetric,dense>&
				dest)
{
#define hess_aux dense_aux
@<Compute Hessian for log barrier@>;
#undef hess_aux
}
matrix<T,symmetric,sparse> &hess(const matrix<T,unstructured,dense>&x, matrix<T,symmetric,sparse>&
				 dest)
{
#define hess_aux sparse_aux
@<Compute Hessian for log barrier@>;
#undef hess_aux
}
@ @<Compute Hessian for log barrier@>=
T t=0;
matrix<T,unstructured,dense> X=x; /* We may resize |x|. */
if(has_t)
{
    t=X(X.rows());@:matrix}{rows@>
    X.resize(X.rows()-1,1);@:matrix}{resize@>
}    
if(fctnal)
{
    T fval=fctnal->eval(X)-t;@:functional}{eval@>
    fctnal->grad(X,aux);@:functional}{grad@>
    fctnal->hess(X,dest);@:functional}{hess@>
    if(has_t)
    {
        aux.resize(aux.rows()+1,1);
	dest.resize(dest.rows()+1,dest.rows()+1);
	aux.entry(aux.rows())=T(-1);
	for(index i=1;i<=dest.cols();++i)
	{
	    dest.entry(dest.rows(),i)=0;
	    dest.entry(i,dest.cols())=0;
	}
    }
    aux/=fval;
    dest/=-fval;
    dest+=outerp(aux,&hess_aux);
    return dest;
}
dest.init(X.rows()+(has_t?1:0),X.rows()+(has_t?1:0));@:matrix}{init@>
dest.fillwith(0);@:matrix}{fillwith@>
fct->eval(X,aux);@:function}{eval@>
fct->jacobian(X,jac);@:function}{jacobian@>
if(has_t)
{
    jac.resize(jac.rows(),jac.cols()+1);
    jac.subm(1,jac.rows(),jac.cols())=matrix<T,unstructured,dense>(jac.rows(),1,T(-1));
}
for(index i=1;i<=aux.rows();++i)@:matrix}{rows@>
{
    jac.subm(i,i,1,jac.cols())/=aux(i)-t;@:matrix}{subm@>
    fct->hess(X,hess_aux,i);@:function}{hess@>
    if(has_t) dest.subm(1,dest.rows()-1,1,dest.cols()-1)-=(hess_aux/=t-aux(i));
    else dest-=(hess_aux/=aux(i));
    dest+=outerp(jac.subm(i,i,1,jac.cols()),&hess_aux);
}
return dest;
