@* The linear combination of functionals. Our next definition is a functional that is
the weighed sum of other specified 
functionals. It will be useful, for example, in optimization methods (when summing barrier
functions).
@c
/* Empty, waiting for |export| */
@ @(functional/linear.h@>=
#ifndef __MATH_LINEAR_FUNCTIONAL__
#define __MATH_LINEAR_FUNCTIONAL__ 1.00
#include <algorithm>
#include <math/functionalbase.h>
namespace math
{
    @[namespace functional@]
    {
	template<class T>@;
	class @[linear : public base<T>@]
	{
	    @<Linear combination of functionals internal variables@>@;
	public:@;
	    @<Linear combination of functionals class methods@>@;
	};
    }
}
#endif
@ The functionals to be added are stored in a vector of pairs of weights and functionals.
@<Linear combination of functionals internal variables@>=
typedef std::pair<T,base<T>*> element_type;
vector<element_type> elements;
typedef typename vector<element_type>::iterator iterator;
@ Adding elements to the sum is a matter of passing weights and functional pointers to the |add|
function. We 
also allow the user to specify initial values at the time of construction.
@<Linear combination of functionals class methods@>=
linear(void)@+{@+}
linear(base<T>* elment,T weight=1)@+
{@+
    add(elment,weight);@+
}
linear(const vector<element_type>& elem)@+
{@+
    elements=elem;@+
}

void add(base<T>* elment,T weight=1)
{
    if(weight==0) return;
    elements.push_back(element_type(weight,elment));
}
void add(const linear<T>* elment,T weight=1)
{
    for(int i=0;i!=elment->size();++i)
        add(elment->get_term(i),weight*elment->get_weight(i));
}
@ We can remove functionals from the end of the list.
@<Linear combination of functionals class methods@>=
void pop_back(void)
{
    elements.pop_back();
}
@ We will need to modify weights 
@<Linear combination of functionals class methods@>=
void set_weight(int i,T value)@+
{@+
    elements[i].first=value;@+
}
T get_weight(int i) const@+
{@+return elements[i].first;@+}
void set_term(int i,base<T>* value)@+
{@+ elements[i].second=value;@+}
base<T>* get_term(int i) const@+
{@+return elements[i].second;@+}
int size(void) const@+
{@+return elements.size();@+}
@ The pointers will be erased, but not the functionals they point to, when the |linear| is deleted.
@<Linear combination of functionals class methods@>=
~linear(void)@+
{@+
    elements.erase(elements.begin(),elements.end());@+
}
@ Now to evaluation. Pretty simple, as you may expect. The default behavior is to return zero if
there are no functionals in the list.
@<Linear combination of functionals class methods@>=
T eval(const matrix<T,unstructured,dense>& x)
{
    T result=0;
    for(iterator i=elements.begin();
	i!=elements.end();++i)
    result+= i->first * i->second->eval(x);@:functional}{eval@>
    return result;
}
@ For the gradient, the default when no elements are present is also to return zero. We set up an 
internal variable |aux| so that we don't need to create/resize vectors each time a new gradient
is computed. Also, we test for unity weights in order to save computation.
@<Linear combination of functionals internal variables@>=
matrix<T,unstructured,dense> aux;
@ @<Linear combination of functionals class methods@>=
matrix<T,unstructured,dense>& grad(const matrix<T,unstructured,dense>& x,
				   matrix<T,unstructured,dense>& dest)
{
    if(elements.size()==0)
    {
	dest.resize(x.rows(),1);@:matrix}{resize@>@:matrix}{rows@>
	dest.fillwith(0.0);@:matrix}{fillwith@>
	return dest;
    }
    iterator i=elements.begin();
    i->second->grad(x,dest);@:functional}{grad@>
    if(i->first!=1) dest*=i->first;@:matrix}{*=@>
    for(i++;i!=elements.end();++i)
    {
        i->second->grad(x,aux);
	if(i->first!=1) aux*=i->first;
	dest+=aux;
    }
    return dest;
}
@ We follow the same algorithm for the Hessian, again testing for unity weights in order to save
computation.
@<Linear combination of functionals internal variables@>=
matrix<T,symmetric,dense> Hdense;
matrix<T,symmetric,sparse> Hsparse;
@ @<Linear combination of functionals class methods@>=
matrix<T,symmetric,dense> &hess(const matrix<T,unstructured,dense>&x, matrix<T,symmetric,dense>&
				dest)
{
    if(elements.size()==0)
    {
	dest.resize(x.rows(),x.rows());@:matrix}{resize@>@:matrix}{rows@>
	dest.fillwith(0.0);@:matrix}{fillwith@>
	return dest;
    }
    iterator i=elements.begin();
    i->second->hess(x,dest);@:functional}{hess@>
    if(i->first!=1) dest*=i->first;@:matrix}{*=@>
    for(i++;i!=elements.end();++i)
    {
        i->second->hess(x,Hdense);
	if(i->first!=1) Hdense*=i->first;
	dest+=Hdense;
    }
    return dest;
}
matrix<T,symmetric,sparse> &hess(const matrix<T,unstructured,dense>&x, matrix<T,symmetric,sparse>&
				dest)
{
    if(elements.size()==0)
    {
	dest=@[matrix<T,symmetric,sparse>@](x.rows(),x.rows());@:matrix}{rows@>
	return dest;
    }
    iterator i=elements.begin();
    i->second->hess(x,dest);@:functional}{hess@>
    if(i->first!=1) dest*=i->first;@:matrix}{*=@>
    for(i++;i!=elements.end();++i)
    {
        i->second->hess(x,Hsparse);
	if(i->first!=1) Hsparse*=i->first;
	dest+=Hsparse;
    }
    return dest;
}
