@* The norm-2 error. This functional computes, for a given function~$f$ and a vector~$y$, the
value of $$\Vert y_i-f_i(x)\Vert^2.$$ This is an extremely useful functional for optimization
problems: suppose you trying to approximate some set of data~$y$ to a function~$f$ that depends
on the {\it parameters\/}~$x$ -- then minimizing this functional is what you want to do. Note the 
confusing naming (of which we cannot escape). Normally the set of data is~$(x_i,y_i)$, but in our 
case the~$x_i$ is stored in the function~$f$ itself -- they are not the argument to~$f$.
@c
/* Empty, waiting for |export|. */
@ @(functional/norm2err.h@>=
#ifndef __MATH_NORM2ERR_FUNCTIONAL__
#define __MATH_NORM2ERR_FUNCTIONAL__
#include<math/algebra.h>
#include<math/function.h>
#include<math/functionalbase.h>
namespace math
{
    @[namespace functional@]
    {
	template<class T,template<@[class@]> class storage>@;
	class @[norm2err : public base<T>@]
	{
	    @<Norm-2 error functional internal variables@>@;
	public:@;
	    @<Norm-2 error functional class methods@>@;
	};
    }
}
#endif
@ The internal matrix variables are the vector~$y$, two auxiliar vectors for computing the value
and gradient of the function, and an auxiliar matrix for computing the Hessian. The other
internal variable is the approximating function.
@<Norm-2 error functional internal variables@>=
matrix<T,unstructured,dense> Y,aux,fval;
matrix<T,symmetric,storage> H;
function::base<T>* F;
@ We can change all the values at any time or at construction.
@<Norm-2 error functional class methods@>=
matrix<T,unstructured,dense>& y(void)@+{@+ return Y; @+}
function::base<T>* &f(void)@+{@+ return F; @+}
norm2err(const matrix<T,unstructured,dense>& newy, function::base<T>* newf=0) : Y(newy),
F(newf)@+ {}
norm2err(function::base<T>* newf=0) : F(newf)@+
{}
@ Now to the eval function.
@<Norm-2 error functional class methods@>=@!@:functional::norm2err}{eval@>
virtual T eval(const matrix<T,unstructured,dense>& x)
{
    fval=F->eval(x,&fval);@:function}{eval@>
    fval-=Y;
    return dot(fval,fval);
}
@ The gradient is given by $$2Df(x)^T(f-y)$$
@<Norm-2 error functional class methods@>= @!@:functional::norm2err}{grad@>
virtual matrix<T,unstructured,dense>& grad(const matrix<T,unstructured,dense>& x,
					   matrix<T,unstructured,dense>& dest)
{
    fval=F->eval(x,&fval);@:function}{eval@>
    fval-=Y;
    F->jacobian(x,&aux);@:function}{jacobian@>
    atxmul(aux,fval,&dest);
    dest*=2;
    return dest;
}
@ The Hessian is a little bit more complicated. It is given by
$$2\sum_i(f_i(x)-y_i)\nabla^2f_i(x) + 2 Df(x)^TDf(x).$$ We have all the
necessary algebraic functions already defined, though.
@<Norm-2 error functional class methods@>=@!@:functional::norm2err}{hess@>
virtual matrix<T,symmetric,dense>& hess(const matrix<T,unstructured,dense>&x ,
					matrix<T,symmetric,dense>& dest)
{
    outterp(F->jacobian(x,&aux),&dest);@:function}{jacobian@>
    fval=F->eval(x,&fval);@:function}{eval@>
    fval-=Y;
    for(index i=1;i<=Y.rows();++i)@:matrix}{rows@>
    {
	H=F->hess(x,&H,i);@:function}{hess@>
	H*=fval(i);
	dest+=H;
    }
    dest*=2;
    return dest;
}
@ @<Norm-2 error functional class methods@>=@!@:functional::norm2err}{hess@>
virtual matrix<T,symmetric,sparse>& hess(const matrix<T,unstructured,dense>&x ,
					matrix<T,symmetric,sparse>& dest)
{
    outterp(F->jacobian(x,aux),&dest);@:function}{jacobian@>
    fval=F->eval(x,&fval);@:function}{eval@>
    fval-=Y;
    for(index i=1;i<=Y.rows();++i)@:matrix}{rows@>
    {
	H=F->hess(x,&H,i);@:function}{hess@>
	H*=fval(i);
	dest+=H;
    }
    dest*=2;
    return dest;
}
