@** Functions. Functions are things that take a matrix as argument and return a matrix. We will
define them before functionals (which take vectors and return scalars) because, for optimization
purposes, functionals are more useful but they need functions in order to be of use. Here we
define the basic interface in a base class from which specific functionals will be derived.
@s function int
@c
/* Not until |export| */
@ @(functionbase.h@>=
#ifndef __MATH_FUNCTION__
#define __MATH_FUNCTION__ 1.00
#include <math/math.h>
#include <math/symmetric.h>
#include <math/sparse.h>
namespace math
{
    @[namespace @!function@]
    {
	template<class T>@;
	class base
	{
	public:@/
	    @<Function base class methods@>@;
	};
    }
}
#endif
@ The first thing we do is to declare a virtual destructor, so that we can 
derive classes from the base class.
@<Function base class methods@>=
virtual ~base(void)@+ {}
@ Before defining the interface we need to consider some points about functions (this rant will
be repeated when defining functionals): first, a function should be able to get any kind of
matrix as the point, that is, the~$x$ in~$f(x)$ could be sparse, dense and so on. Also, we should
be able to create a list containing various types of functions. The impossibility of this ideal
situation is summarized by Stroustrup: ``A member template cannot be virtual.'' This is a design
decision, and a wrong one in my point of view. We have then only two possibilities: either we
make the |matrix| class derived, so we can pass pointers, or we fix the type of the parameter.
The first option results in performance degradation, so we're stuck with the second. Fortunately
most functions take vectors as arguments, so that matrix structures are not a big deal. We will
assume that |dense| vectors are the best compromise. Also, we fix the return value to a |dense|
and |unstructured| matrix because we consider that in most cases we'll be returning vectors.
Storage for the result must be provided by the user.
@<Function base class methods@>=
matrix<T,unstructured,dense>& operator()(const matrix<T,unstructured,dense>&
x,matrix<T,unstructured,dense>* dest) {@+ return eval(x,dest);@+ }
virtual matrix<T,unstructured,dense>& eval(const matrix<T,unstructured,dense>&
x,matrix<T,unstructured,dense>* dest) = 0;
@ If possible, a function should compute the Jacobian. If~$f:\reals[n]\to\reals[m]$, then the
Jacobian is a matrix $J\in\reals[m][n]$ defined by~$J_{ij}=df_i/dx_j$. Here we face another
interface 
decision: functions are supposed to be used to compute lots of points, that is, when you create a
function, usually you want to evaluate it in a set of points, not just only one. If a evaluation
method created storage for the result each time it was called, then we would be facing a serious
performance threat. For this reason we require the user to pass the result variable as an
argument. This requirement, in turn, makes the type of the result matrix fixed for the same
reasons discussed above. The method returns the result (that is, |dest|), which is a useful
behavior in certain applications, as for example in function compositions like~$f(g(x))$. The
size of the |dest| matrix at input {\em should be checked by the method}. This will be normally
performed via the |matrix::resize| method. Since this method checks for same dimensions, the
performance does not suffer too much.
@<Function base class methods@>=@!@:function::base}{grad@>
virtual matrix<T,unstructured,dense>&
jacobian(const matrix<T,unstructured,dense>& x, matrix<T,unstructured,dense>* dest) = 0;
@ The same thing is valid for the Hessian, except here we require the Hessian to be
symmetric (which it is by definition). The same remarks about |dest| size made above are
valid here. We provide two methods, one for |sparse| matrices (for large optimization problems it 
may be crucial to get a sparse version).
@<Function base class methods@>=@!@:function::base}{hess@>
virtual matrix<T,symmetric,dense>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,dense>* dest, const index i, const 
index j=1)  = 0;
virtual matrix<T,symmetric,sparse>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,sparse>* dest, const index i,const 
index j=1)  = 0;
