@* The bisection algorithm. Although not really a true bisection algorithm, it works by halving
the step size when appropriate as we will see. This is only an example of line searching, but
this algorithm can fail miserably in practice: depending on the function and on the disposition
of local minima, you can end up in a local minima that is bigger than the closest one. You're
advised do use other methods if possible. That said, we move on: As with many algorithms, we must
provide a stopping criterion. In this case, the algorithm stops whenever~$\Vert x-x^*\Vert\leq
tol$, where $tol$ is a given tolerance. Also, some people claim that halving the step size is not
always the best thing to do. We then provide a means to alter this behavior via a |ratio|
parameter, so that when the step size is changed, it changes via |step=step*ratio|. Of course, we
need |0<ratio<1| for the algorithm to work.
@c
/* Empty, waiting for |export|. */
@ @(linesearch/bisection.h@>=
#ifndef __MATH_BISECTION__
#define __MATH_BISECTION__ 1.0
#include<math.h>
const double bisection_infty=HUGE_VAL; /* Some compilers have problems inlining this constant. */
#include<math/algebra.h>
#include<math/linesearchbase.h>
namespace math
{
    @[namespace linesearch@]
    {
	template<class T>@;
	class @[bisection : public base<T>@]
	{
	    double tol,ratio;
	    unsigned long maxiter; /* Stop in case its unbounded below or something. */
	public:
	    ~bisection(void){}
	    @<Bisection line search methods@>@;
	};
	@<|bisection| big definitions@>;
    }
}
#endif
@ First we provide a constructor that enable us to set all parameters at creation time, with some 
defaults in case we don't care about it.
@<Bisection line search methods@>= 
bisection(double t=1e-3, double r=0.5, unsigned long i=1000)
: tol(t), ratio(r), maxiter(i) @+{@+}
@ The first thing we do is to define the method that takes the gradient as an argument. We don't
use the gradient for this algorithm, so the only thing we do is to call the other method.
@<Bisection line search methods@>=@!@:bisection}{minimize@>
matrix<T,unstructured,dense>
minimize(functional::base<T>* f,const matrix<T,unstructured,dense>& x0,const
	 matrix<T,unstructured,dense>& g0, const matrix<T,unstructured,dense>& dir)
{
    return minimize(f,x0,dir);@:linesearch::bisection}{minimize@>
}
@ The algorithm works as follows: we make |x=x0|. At each step, we start from $x$ and go in the
descending direction until we pass the minimum, as indicated by an increase in the function. At
this point, we know the descent direction will be to the other side, so we turn around and
proceed slower, that is, with a smaller stepsize. The stopping criterion is independent of~$x^*$
and~$x$ since $$\Vert x-x^*\Vert\leq\Vert x-(x+\delta x)\Vert=\Vert\delta x\Vert,$$ where
$\Vert\delta x\Vert$ is a positive multiple of $\Vert dir\Vert$. We consider that reaching the
maximum number of iterations is an error. Note that the maximum number of iterations is with
respect to the {\it inner\/} loop, since the outer loop always finishes. The algorithm follows:
@<Bisection line search methods@>=@!@:bisection}{minimize@>
matrix<T,unstructured,dense>
minimize(functional::base<T>* func,const matrix<T,unstructured,dense>& x0,
	 const matrix<T,unstructured,dense>& dir);
@ @<|bisection| big definitions@>=
template<class T> matrix<T,unstructured,dense>
bisection<T>::minimize(functional::base<T>* func,const matrix<T,unstructured,dense>& x0,
	const matrix<T,unstructured,dense>& dir)
{
    double stepsize=1, dirnorm=norm2(dir);
    matrix<T,unstructured,dense> x(x0);
    unsigned long outer_iter=0;
    while(fabs(stepsize/ratio)*dirnorm>tol && ++outer_iter<=maxiter)
    {
	double f=func->eval(x);
	double newf;
	@<Compute next function value for |bisection|@>;
	unsigned long inner_iter=0;
	while(newf<f)
	{
	    f=newf;
	    @<Compute next function value for |bisection|@>;
	    if(++inner_iter>maxiter) throw error::unboundedbelow();
	}
	stepsize*=-ratio;
    }
    return x;
}
@ @<Compute next function value for |bisection|@>=
saxpy(stepsize,dir,&x);
try { newf=func->eval(x); }
catch(error::domain e) 
{ newf=bisection_infty; }
while(!finite(newf))
{
    saxpy(-stepsize,dir,&x);
    stepsize*=ratio;
    saxpy(stepsize,dir,&x);
    try { newf=func->eval(x); }
    catch(error::domain e) 
    { newf=bisection_infty; }
}
if(stepsize==0) return x;
