@* Enforcing equality constraints. All the search direction classes defined above are not able to
take into account the search direction must, in some cases, have zero component in some
directions. The most obvious and common case is when we want to minimize something while
enforcing an equality constraint~$Ax=b$. We will define a class that, based on a search
direction~$d$ computed in any way, transforms it so that~$A(x+tv)=b$ for any real~$t$.
@c
/* Empty, waiting for |export|. */
@ @(searchdir/equality.h@>=
#ifndef __MATH_SEARCHDIR_EQUALITY__
#define __MATH_SEARCHDIR_EQUALITY__ 1.0
#include<math/searchdirbase.h>
#include<math/symmetric.h>
#include<math/lu.h>
namespace math
{
    @[namespace searchdir@]
    {
	template<matrix_simple_template>@;
	class @[equality : public base<T>@]
	{
	    @<Equality search direction internal variables@>@;
	public:@;
	    @<Equality search direction methods@>@;
	};
    }
}
#endif
@ First we need a  way to let the user define which basic search direction he wants to use.
@<Equality search direction internal variables@>=
base<T>* dirf;
@ @<Equality search direction methods@>=
equality(void)@[:@]dirf(0)@+{}
equality(base<T>* newf) @[:@] dirf(newf)@+{}
base<T>*& f(void)@+
{@+
    return dirf;@+
}
@ Now to the~$A$ matrix. Suppose that our basic direction class provides us
with a preferred direction~$d_0$. What we do is to compute the projection of~$d_0$ into the
kernel of~$A$. This task can be acomplished by solving the linear system $$\bmatrix{I & A^T \cr A 
& 0}\bmatrix{d\cr w}=\bmatrix{d_0 \cr 0},$$ which in the end will give $$d=(I-A^T\inv(AA^T)A)d_0$$.
That's the only time where we use the~$A$ matrix, so
we can see that we don't need to store it: what we store is the LU decomposition (the matrix is
not necessarily positive-definite) of the
linear system matrix, so that no matter how many directions we compute, solving the linear system 
is a very fast operation (no decompositions needed!). The only catch here is that we may receive
(and we will when barrier functions are defined) a preferred direction with the wrong dimension.
We store the correct dimension in |xrows| and deal with it later on.
@<Equality search direction internal variables@>=
matrix<T,unstructured,storage> decomp;
vector<index> pivots;
index xrows;
@ @<Equality search direction methods@>=
equality(const matrix<T,structure,storage>& A)@[:@]
dirf(0)@+{@+ set_A(newa);@+}
void set_A(const matrix<T,structure,storage>& A)
{
#ifdef __MATH_DEBUG__
    cout << "[math]: equality search constructor.\n";
    for(index i=1;i<=A.rows();++i)
    {
	for(index j=1;j<=A.cols();++j) cout << A(i,j) << ' ';
	cout << '\n';
    }
#endif
    xrows=A.cols();
    decomp.resize(A.cols()+A.rows(),A.cols()+A.rows());@:matrix}{cols@>@:matrix}{rows@>
    decomp.fillwith(0.0);@:matrix}{fillwith@>@:matrix}{resize@>
    decomp.subm(1,A.cols(),1+A.cols(),decomp.cols())=transpose(A);@:matrix}{subm@>
    decomp.subm(1+A.cols(),decomp.rows(),1,A.cols())=A;@:matrix}{rows@>
    for(index i=1;i<=A.cols();++i) decomp.entry(i,i)=1;@:matrix}{entry@>
    lu::decompose(&decomp,&pivots);@:lu}{decompose@>
}
@ Now we're ready to compute the direction. As said before, we may receive a preferred direction
with the wrong dimensions. This happens when barrier functions are in use and a new slack variable
is introduced in the problem after the~|A| matrix is defined. Therefore, when~|x| has the wrong
dimension, we know it has exactly one more component than the original number, and what we do is
to project only the first components.
@ @<Equality search direction methods@>=
matrix<T,unstructured,dense>&
dir(functional::base<T>* f,const matrix<T,unstructured,dense>& x,
	matrix<T,unstructured,dense>* dest)
{
    double extra_component=0;
    dest->resize(decomp.rows(),1);@:matrix}{rows@>@:matrix}{resize@>
    dest->fillwith(0.0);@:matrix}{fillwith@>
    dirf->dir(f,x,dest);@:searchdir}{dir@>@:matrix}{subm@>
#ifdef __MATH_DEBUG__
cout << "[math]: equality search dir begin.\n[math]: orig dir = ";
for(index i=1;i<=x.rows();++i)
    cout << (*dest)(i) << ' ';
cout << '\n';
#endif
    if(x.rows()!=xrows) extra_component=(*dest)(xrows+1);
    dest->entry(xrows+1)=0.0;
    lu::finish(decomp,pivots,dest);@:lu}{finish@>
    dest->resize(x.rows(),1);
    if(x.rows()!=xrows) dest->entry(xrows+1)=extra_component;
#ifdef __MATH_DEBUG__
cout << "[math]: new dir = ";
for(index i=1;i<=result.rows();++i)
    cout << (*dest)(i) << ' ';
cout << "\n[math]: equality search dir end.\n";
#endif
    return *dest;
}
